package net.pricefx.common.apibuilder.chart;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.api.chart.WaterfallColumnType;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;

public class WaterfallChart extends AbstractBuilder {

	public WaterfallChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}


	private WaterfallOptions options;
	private WaterfallSeries series;
	private Dictionary<WaterfallChart> dictionary;
	
	public WaterfallSeries getSeries() {
		String id = addInvocation("getSeries");
		if (series == null) series = new WaterfallSeries(this, id, "Series", idGenerator, invocations);
		return series;
	}
	
	public WaterfallOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new WaterfallOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public Dictionary<WaterfallChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.WATERFALL;
		return r;
	}

	
	public static class WaterfallOptions extends AbstractOptions<WaterfallChart,WaterfallOptions> {

		public WaterfallOptions(WaterfallChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}
	}
	
	public static class WaterfallSeries extends AbstractSeries<WaterfallChart,WaterfallSeries> {

		public WaterfallSeries(WaterfallChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}

		private Aggregation<WaterfallSeries> aggregation;
		
		public WaterfallSeries setHideDataLabels(Boolean hideDataLabels) {

			addInvocation("setHideDataLabels",hideDataLabels);
			return this;
		}

		public Aggregation<WaterfallSeries> getAggregation() {
			
			String id = addInvocation("getAggregation");
			if (aggregation == null) 
				aggregation = new Aggregation<WaterfallSeries>(this, id, "Aggregation", idGenerator, invocations);
			return aggregation;
		}
		
		public WaterfallSeries addWaterfallColumn(WaterfallColumnType columnType, String source, String label) {

			addInvocation("addWaterfallColumn",columnType, source, label);
			return this;
		}
		
		public Formula<WaterfallSeries> setFormula() {

			String id = addInvocation("setFormula");
			return new Formula<WaterfallSeries>(this, id, "Formula");
		}
		
		public WaterfallSeries setGeneratedQueryDto(String generatedQueryDto) {
			
			addInvocation("setGeneratedQueryDto",generatedQueryDto);
			return this;
		}
	}
	
	public static class Aggregation<ATTACHEDTO extends ObjectIdentity> extends AggrFieldAxis<ATTACHEDTO> {

		public Aggregation(ATTACHEDTO attachedTo, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(attachedTo, instanceId, classId);
		}
		
	}
	
	
}