package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.common.api.InputType;

/**
 * Builder class for input type StringUserEntry {@link InputType#STRINGUSERENTRY}.
 */
public class StringInputBuilder<VT> extends AbstractInputBuilder<StringInputBuilder, VT> {

    protected StringInputBuilder(InputBuilderContext builderContext, String paramName) {
        super(builderContext, InputType.STRINGUSERENTRY, paramName);
    }

    /**
     * Sets text mask on ContextParameter, used in the dropdown input field.
     * <p>Example:
     * <blockquote><pre>
     * api.inputBuilderFactory()
     *      .createStringUserEntry("StringEntry")
     *      .setTextMask("Enter string value ...")
     *      .setLabel("A string entry without refresh")
     *      .setNoRefresh(true)
     *      .buildContextParameter()
     * </pre></blockquote>
     *
     * @param textMask text mask
     * @return self
     */
    public StringInputBuilder setTextMask(String textMask) {
        cp.setValueOptions(new String[]{textMask});
        return this;
    }
}
