package net.pricefx.workflowengine;

import java.util.Set;

import net.pricefx.common.api.WorkflowType;
import net.pricefx.common.dto.WorkflowDTO.ApprovalStepDTO;
import net.pricefx.common.dto.WorkflowDTO.WatcherStepDTO;
import net.pricefx.domain.User;

public interface Workflow {

	String ID = "id";
	String TYPE = "type";
	String APPROVABLE_TYPED_ID = "approvableTypedId";
	String WORKFLOW_STATUS = "workflowStatus";
	String SUBMITTER_TYPED_ID = "submitterTypedId";
	String SUBMITTER_LOGIN_NAME = "submitterLoginName";
	String SUBMIT_REASON = "submitReason";
	String DENIAL_REASONS = "denialReasons";
	String IS_DENIAL_REASON_OPTIONAL = "isDenialReasonOptional";
	String IS_ADD_STEP_RESTRICTED_TO_WF_ADMIN = "isAddStepRestrictedToWFAdmin";
	String IS_SELECT_STEP_ON_DENY = "isSelectStepOnDeny";
	String SELECTED_STEP_ON_DENY = "selectedStepOnDeny";
	String DATA_MAP = "dataMap";
	String DEFAULT_POST_APPROVAL_STEP_LOGIC_NAME = "defaultPostApprovalStepLogicName";
	String AUTO_CONVERT_TO_DEAL = "autoConvertToDeal";

	String getId();
	WorkflowType getType();
	
	void addApprovalStep(String previousStepId, ApprovalStepDTO approvalStepToAdd, boolean calledFromPostStepLogic);
	void addWatcherStep(String precedingStepId, WatcherStepDTO watcherStepToAdd, boolean calledFromPostStepLogic);
	
	void approve(String stepId, String comment, String selectedApprover);
	void deny(String stepId, String comment, String selectedApprover, String selectedStep);
	void submit();
	void withdraw(String stepId, String comment);
	void invalidate();
	WorkflowStatus getStatus();
	
	Set<String> getUsers();
	Set<String> getActiveUsers();

	String getSubmitterTypedId();
	String getApprovableTypedId();
	String getCurrentStepId();
	String getVersion();
	String getDefaultPostApprovalStepLogicName();
	
	void setSubmitter(User submitter);
	
	WorkflowInfo getWorkflowInfo();

	void setExternalTokenRegenerationCallback(Object callback);

	boolean writeNamesToWorkflowDefinition();

	public enum WorkflowStatus {
		DRAFT,
		SUBMITTED,
		DENIED,
		APPROVED,
		/** 
		 * Special case, engine is bypassed
		 */
		NO_APPROVAL_REQUIRED,
		WITHDRAWN,
		/**
		 * Special case when system automatically invalidates (withdraws) the workflow
		 * At the moment, it only happens when the bound approvable gets invalidated
		 * @see Workflow#invalidate()
		 */
		INVALIDATED
	}
}
